changeset 33150:9d06314be6ab

memcoll: clarify sizes versus lengths, document better, and tweak perf * lib/memcoll.c (strcoll_loop, memcoll0): Improve quality of descriptive comments. Name variables consistently as to whether they are lengths (which do not include terminating null) versus sizes (which do). * lib/xmemcoll.c (xmemcoll0): Likewise. * lib/memcoll.c (strcoll_loop): Tweak the way that the diff is returned when s1size == 0; this is easier to compile and saves about 17% of memcoll's code space on x86-64 with GCC 4.1.2.
author Paul R. Eggert <eggert@cs.ucla.edu>
date Mon, 12 Jul 2010 10:58:35 -0700
parents 19b2b7c435bf
children 4cd6774967ec
files ChangeLog lib/memcoll.c lib/xmemcoll.c
diffstat 3 files changed, 39 insertions(+), 29 deletions(-) [+]
line wrap: on
line diff
--- a/ChangeLog	Mon Jul 12 18:55:19 2010 +0200
+++ b/ChangeLog	Mon Jul 12 10:58:35 2010 -0700
@@ -1,3 +1,15 @@
+2010-07-12  Paul R. Eggert  <eggert@cs.ucla.edu>
+
+	memcoll: clarify sizes versus lengths, document better, and tweak perf
+	* lib/memcoll.c (strcoll_loop, memcoll0):
+	Improve quality of descriptive comments.  Name variables
+	consistently as to whether they are lengths (which do not include
+	terminating null) versus sizes (which do).
+	* lib/xmemcoll.c (xmemcoll0): Likewise.
+	* lib/memcoll.c (strcoll_loop): Tweak the way that the diff is
+	returned when s1size == 0; this is easier to compile and saves
+	about 17% of memcoll's code space on x86-64 with GCC 4.1.2.
+
 2010-07-12  Bruno Haible  <bruno@clisp.org>
 
 	Tests for module '_Exit'.
--- a/lib/xmemcoll.c	Mon Jul 12 18:55:19 2010 +0200
+++ b/lib/xmemcoll.c	Mon Jul 12 10:58:35 2010 -0700
@@ -60,8 +60,10 @@
   return diff;
 }
 
-/* Like xmemcoll, but S1 and S2 are known to be NUL delimited, thus
-   no modifications to S1 and S2 are needed. */
+/* Compare S1 (with size S1SIZE) and S2 (with length S2SIZE) according
+   to the LC_COLLATE locale.  S1 and S2 are both blocks of memory with
+   nonzero sizes, and the last byte in each block must be a null byte.
+   Report an error and exit if there is an error.  */
 
 int
 xmemcoll0 (char const *s1, size_t s1len, char const *s2, size_t s2len)
--- a/lib/memcoll.c	Mon Jul 12 18:55:19 2010 +0200
+++ b/lib/memcoll.c	Mon Jul 12 10:58:35 2010 -0700
@@ -26,11 +26,13 @@
 #include <stdlib.h>
 #include <string.h>
 
-/* Ensure strcoll operates on the entire input strings, in case they contain
-   NUL bytes. */
-
+/* Compare S1 (with size S1SIZE) and S2 (with length S2SIZE) according
+   to the LC_COLLATE locale.  S1 and S2 are both blocks of memory with
+   nonzero sizes, and the last byte in each block must be a null byte.
+   Set errno to an error number if there is an error, and to zero
+   otherwise.  */
 static inline int
-strcoll_loop (char const *s1, size_t s1len, const char *s2, size_t s2len)
+strcoll_loop (char const *s1, size_t s1size, char const *s2, size_t s2size)
 {
   int diff;
 
@@ -43,20 +45,13 @@
       size_t size2 = strlen (s2) + 1;
       s1 += size1;
       s2 += size2;
-      s1len -= size1;
-      s2len -= size2;
+      s1size -= size1;
+      s2size -= size2;
 
-      if (s1len == 0)
-        {
-          if (s2len != 0)
-            diff = -1;
-          break;
-        }
-      else if (s2len == 0)
-        {
-          diff = 1;
-          break;
-        }
+      if (s1size == 0)
+        return - (s2size != 0);
+      if (s2size == 0)
+        return 1;
     }
 
   return diff;
@@ -86,30 +81,31 @@
       char n1 = s1[s1len];
       char n2 = s2[s2len];
 
-      s1[s1len++] = '\0';
-      s2[s2len++] = '\0';
+      s1[s1len] = '\0';
+      s2[s2len] = '\0';
 
-      diff = strcoll_loop (s1, s1len, s2, s2len);
+      diff = strcoll_loop (s1, s1len + 1, s2, s2len + 1);
 
-      s1[s1len - 1] = n1;
-      s2[s2len - 1] = n2;
+      s1[s1len] = n1;
+      s2[s2len] = n2;
     }
 
   return diff;
 }
 
-/* Compare S1 (with length S1LEN) and S2 (with length S2LEN) according
-   to the LC_COLLATE locale.  S1 and S2 must both end in a null byte.
+/* Compare S1 (with size S1SIZE) and S2 (with length S2SIZE) according
+   to the LC_COLLATE locale.  S1 and S2 are both blocks of memory with
+   nonzero sizes, and the last byte in each block must be a null byte.
    Set errno to an error number if there is an error, and to zero
    otherwise.  */
 int
-memcoll0 (char const *s1, size_t s1len, const char *s2, size_t s2len)
+memcoll0 (char const *s1, size_t s1size, char const *s2, size_t s2size)
 {
-  if (s1len == s2len && memcmp (s1, s2, s1len) == 0)
+  if (s1size == s2size && memcmp (s1, s2, s1size) == 0)
     {
       errno = 0;
       return 0;
     }
   else
-    return strcoll_loop (s1, s1len, s2, s2len);
+    return strcoll_loop (s1, s1size, s2, s2size);
 }